@@ -7,8 +7,9 @@ use Plack::Response;
 use AnyEvent::WebSocket::Server;
 use Try::Tiny;
 use Plack::App::WebSocket::Connection;
+use Scalar::Util qw(blessed);
 
-our $VERSION = "0.04";
+our $VERSION = "0.05";
 
 my $ERROR_ENV = "plack.app.websocket.error";
 
@@ -19,8 +20,14 @@ sub new {
     croak "on_establish param must be a code-ref" if ref($self->{on_establish}) ne "CODE";
     $self->{on_error} ||= \&_default_on_error;
     croak "on_error param must be a code-ref" if ref($self->{on_error}) ne "CODE";
+
+    if(!defined($self->{websocket_server})) {
+        $self->{websocket_server} = AnyEvent::WebSocket::Server->new();
+    }
+    if(blessed($self->{websocket_server}) && !$self->{websocket_server}->isa("AnyEvent::WebSocket::Server")) {
+        croak "websocket_server param must be a AnyEvent::WebSocket::Server";
+    }
     
-    $self->{websocket_server} = AnyEvent::WebSocket::Server->new();
     return $self;
 }
 
@@ -65,9 +72,14 @@ sub call {
         my $responder = shift;
         $cv_conn->cb(sub {
             my ($cv_conn) = @_;
-            my ($conn) = try { $cv_conn->recv };
+            my ($conn, $error) = try {
+                (scalar($cv_conn->recv), undef);
+            }catch {
+                (undef, $_[0]);
+            };
             if(!$conn) {
                 $env->{$ERROR_ENV} = "invalid request";
+                $env->{"plack.app.websocket.error.handshake"} = $error;
                 _respond_via($responder, $self->{on_error}->($env));
                 return;
             }
@@ -188,6 +200,11 @@ that briefly describes the error (See below).
 By default, it returns a simple non-200 HTTP response according to C<< $psgi_env->{"plack.app.websocket.error"} >>.
 See below for detail.
 
+=item C<websocket_server> => L<AnyEvent::WebSocket::Server> (optional)
+
+The backend L<AnyEvent::WebSocket::Server> instance.
+By default, C<< AnyEvent::WebSocket::Server->new() >> is used.
+
 =back
 
 =head1 C<plack.app.websocket.error> ENVIRONMENT STRINGS
@@ -206,6 +223,7 @@ By default, 500 "Internal Server Error" response is returned for this error.
 =item C<"invalid request">
 
 The client sent an invalid request.
+In this case, C<< $psgi_env->{"plack.app.websocket.error.handshake"} >> keeps the exception thrown by the handshake process.
 
 By default, 400 "Bad Request" response is returned for this error.
 
